package cd.casic.ci.process.process.dal.label;

import cd.casic.ci.process.process.dataObject.TPipelineLabelPipelineRecord;
import cd.casic.ci.process.process.pojo.PipelineLabelRelateInfo;
import cn.hutool.core.lang.Pair;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * 流水线和标签对应关系Mapper接口
 */
@Repository
public interface PipelineLabelPipelineDao {

    /**
     * 创建流水线标签关系
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @param labelId 标签ID
     * @param userId 用户ID
     * @param id 主键ID
     * @return 影响的行数
     */
    int create(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId,
        @Param("labelId") Long labelId,
        @Param("userId") String userId,
        @Param("id") Long id
    );
    /**
     * 批量创建流水线标签关系
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @param pipelineLabelRels 流水线标签关系列表
     * @param userId 用户ID
     * @return 影响的行数
     */
    int batchCreate(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId,
        @Param("pipelineLabelRels") List<Pair<Long, Long>> pipelineLabelRels,
        @Param("userId") String userId
    );

    /**
     * 删除流水线标签关系
     *
     * @param projectId 项目ID
     * @param id 主键ID
     * @param userId 用户ID
     * @return 影响的行数
     */
    int delete(
        @Param("projectId") String projectId,
        @Param("id") Long id,
        @Param("userId") String userId
    );

    /**
     * 删除流水线的所有标签关系
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @param userId 用户ID
     * @return 影响的行数
     */
    int deleteByPipeline(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId,
        @Param("userId") String userId
    );

    /**
     * 删除标签的所有流水线关系
     *
     * @param projectId 项目ID
     * @param labelId 标签ID
     * @param userId 用户ID
     * @return 影响的行数
     */
    int deleteByLabel(
        @Param("projectId") String projectId,
        @Param("labelId") Long labelId,
        @Param("userId") String userId
    );

    /**
     * 获取标签关联的流水线列表
     *
     * @param projectId 项目ID
     * @param labelId 标签ID集合
     * @return 流水线标签关系记录列表
     */
    TPipelineLabelPipelineRecord listPipelines(
        @Param("projectId") String projectId,
        @Param("labelId") Set<Long> labelId
    );

    /**
     * 获取流水线关联的标签列表
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @return 流水线标签关系记录列表
     */
    TPipelineLabelPipelineRecord listLabels(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId
    );

    /**
     * 获取已存在标签关系的流水线ID集合
     *
     * @param projectId 项目ID
     * @param pipelineIds 流水线ID集合
     * @return 流水线ID集合
     */
    Set<String> exitsLabelPipelines(
        @Param("projectId") String projectId,
        @Param("pipelineIds") Set<String> pipelineIds
    );

    /**
     * 获取流水线标签关系列表
     *
     * @param pipelineIds 流水线ID集合
     * @param projectId 项目ID
     * @return 流水线标签关系记录列表
     */
    TPipelineLabelPipelineRecord listPipelineLabelRels(
        @Param("pipelineIds") List<String> pipelineIds,
        @Param("projectId") String projectId
    );

    /**
     * 获取流水线标签关联信息列表
     *
     * @param projectIds 项目ID列表
     * @return 流水线标签关联信息列表
     */
    List<PipelineLabelRelateInfo> getPipelineLabelRelateInfos(
        @Param("projectIds") List<String> projectIds
    );
} 