package cd.casic.ci.process.process.dal.label;


import cd.casic.ci.process.process.dataObject.TPipelineViewRecord;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;

/**
 * 流水线视图Mapper接口
 */
@Repository
public interface PipelineViewDao {

    /**
     * 创建视图（基础版）
     *
     * @param projectId 项目ID
     * @param name 视图名称
     * @param isProject 是否项目视图
     * @param filterByPipelineName 按流水线名称过滤
     * @param filterByCreator 按创建者过滤
     * @param userId 用户ID
     * @param id 视图ID（可选）
     * @return 创建的视图ID
     */
    Long create(
        @Param("projectId") String projectId,
        @Param("name") String name,
        @Param("isProject") Boolean isProject,
        @Param("filterByPipelineName") String filterByPipelineName,
        @Param("filterByCreator") String filterByCreator,
        @Param("userId") String userId,
        @Param("id") Long id
    );

    /**
     * 创建视图（高级版）
     *
     
     * @param projectId 项目ID
     * @param name 视图名称
     * @param logic 逻辑表达式
     * @param isProject 是否项目视图
     * @param filters 过滤器
     * @param userId 用户ID
     * @param id 视图ID（可选）
     * @param viewType 视图类型
     * @return 创建的视图ID
     */
    Long create(
        
        @Param("projectId") String projectId,
        @Param("name") String name,
        @Param("logic") String logic,
        @Param("isProject") Boolean isProject,
        @Param("filters") String filters,
        @Param("userId") String userId,
        @Param("id") Long id,
        @Param("viewType") Integer viewType
    );

    /**
     * 更新视图（基础版）
     *
     
     * @param projectId 项目ID
     * @param viewId 视图ID
     * @param name 视图名称
     * @param isProject 是否项目视图
     * @param filterByPipelineName 按流水线名称过滤
     * @param filterByCreator 按创建者过滤
     * @return 是否更新成功
     */
    Boolean update(
        
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId,
        @Param("name") String name,
        @Param("isProject") Boolean isProject,
        @Param("filterByPipelineName") String filterByPipelineName,
        @Param("filterByCreator") String filterByCreator
    );

    /**
     * 更新视图（高级版）
     *
     
     * @param projectId 项目ID
     * @param viewId 视图ID
     * @param name 视图名称
     * @param logic 逻辑表达式
     * @param isProject 是否项目视图
     * @param filters 过滤器
     * @param viewType 视图类型
     * @return 是否更新成功
     */
    Boolean update(
        
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId,
        @Param("name") String name,
        @Param("logic") String logic,
        @Param("isProject") Boolean isProject,
        @Param("filters") String filters,
        @Param("viewType") Integer viewType
    );

    /**
     * 删除视图
     *
     
     * @param projectId 项目ID
     * @param viewId 视图ID
     * @return 是否删除成功
     */
    Boolean delete(
        
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId
    );

    /**
     * 获取项目下的所有视图
     *
     
     * @param projectId 项目ID
     * @return 视图记录列表
     */
    TPipelineViewRecord list(
        
        @Param("projectId") String projectId
    );

    /**
     * 获取项目下指定类型的视图
     *
     
     * @param projectId 项目ID
     * @param viewType 视图类型
     * @return 视图记录列表
     */
    TPipelineViewRecord list(
        
        @Param("projectId") String projectId,
        @Param("viewType") Integer viewType
    );

    /**
     * 获取项目下指定是否项目视图的列表
     *
     
     * @param projectId 项目ID
     * @param isProject 是否项目视图
     * @return 视图记录列表
     */
    TPipelineViewRecord list(
        
        @Param("projectId") String projectId,
        @Param("isProject") Boolean isProject
    );

    /**
     * 获取用户创建的视图列表
     *
     
     * @param projectId 项目ID
     * @param userId 用户ID
     * @return 视图记录列表
     */
    TPipelineViewRecord list(
        
        @Param("projectId") String projectId,
        @Param("userId") String userId
    );

    /**
     * 获取用户创建的指定类型视图列表
     *
     
     * @param projectId 项目ID
     * @param userId 用户ID
     * @param isProject 是否项目视图
     * @return 视图记录列表
     */
    TPipelineViewRecord list(
        
        @Param("projectId") String projectId,
        @Param("userId") String userId,
        @Param("isProject") Boolean isProject
    );

    /**
     * 获取用户视图列表（支持多种过滤条件）
     *
     
     * @param userId 用户ID
     * @param projectId 项目ID
     * @param isProject 是否项目视图（可选）
     * @param viewType 视图类型（可选）
     * @return 视图记录列表
     */
    List<TPipelineViewRecord> list(
        
        @Param("userId") String userId,
        @Param("projectId") String projectId,
        @Param("isProject") Boolean isProject,
        @Param("viewType") Integer viewType
    );

    /**
     * 获取指定ID的视图列表
     *
     
     * @param projectId 项目ID（可选）
     * @param viewIds 视图ID集合
     * @param viewType 视图类型（可选）
     * @return 视图记录列表
     */
    TPipelineViewRecord list(
        
        @Param("projectId") String projectId,
        @Param("viewIds") Collection<Long> viewIds,
        @Param("viewType") Integer viewType
    );

    /**
     * 分页获取视图列表
     *
     
     * @param projectId 项目ID
     * @param isProject 是否项目视图
     * @param viewName 视图名称（可选）
     * @param limit 每页数量
     * @param offset 偏移量
     * @return 视图记录列表
     */
    TPipelineViewRecord listByPage(
        
        @Param("projectId") String projectId,
        @Param("isProject") Boolean isProject,
        @Param("viewName") String viewName,
        @Param("limit") Integer limit,
        @Param("offset") Integer offset
    );

    /**
     * 获取所有项目或用户视图
     *
     
     * @param projectId 项目ID
     * @param isProject 是否项目视图
     * @param userId 用户ID
     * @return 视图记录列表
     */
    TPipelineViewRecord listAll(
        
        @Param("projectId") String projectId,
        @Param("isProject") Boolean isProject,
        @Param("userId") String userId
    );

    /**
     * 获取动态项目ID列表
     *
     
     * @return 项目ID列表
     */
    List<String> listDynamicProjectId();

    /**
     * 获取项目的动态视图列表
     *
     
     * @param projectId 项目ID
     * @return 视图记录列表
     */
    TPipelineViewRecord listDynamicViewByProjectId(
        
        @Param("projectId") String projectId
    );

    /**
     * 获取项目或用户视图列表
     *
     
     * @param projectId 项目ID
     * @param isProject 是否项目视图
     * @param userId 用户ID
     * @return 视图记录列表
     */
    TPipelineViewRecord listProjectOrUser(
        
        @Param("projectId") String projectId,
        @Param("isProject") Boolean isProject,
        @Param("userId") String userId
    );

    /**
     * 获取指定视图
     *
     
     * @param projectId 项目ID
     * @param viewId 视图ID
     * @return 视图记录
     */
    TPipelineViewRecord get(
        
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId
    );

    /**
     * 获取用户创建的指定视图
     *
     
     * @param userId 用户ID
     * @param projectId 项目ID
     * @param viewId 视图ID
     * @return 视图记录
     */
    TPipelineViewRecord get(
        
        @Param("userId") String userId,
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId
    );

    /**
     * 统计同名视图数量
     *
     
     * @param projectId 项目ID
     * @param name 视图名称
     * @param creator 创建者（可选）
     * @param isProject 是否项目视图
     * @param excludeIds 排除的ID集合
     * @return 数量
     */
    Integer countByName(
        
        @Param("projectId") String projectId,
        @Param("name") String name,
        @Param("creator") String creator,
        @Param("isProject") Boolean isProject,
        @Param("excludeIds") Collection<Long> excludeIds
    );
    TPipelineViewRecord fetchAnyByName(
            String projectId,
            String name,
            Boolean isProject
    );


    Integer countForLimit(
            @Param("projectId")String projectId,
            @Param("isProject") Boolean isProject,
            @Param("userId") String userId
    );
} 