package cd.casic.ci.process.process.dal.label;

import cd.casic.ci.process.process.dataObject.TPipelineViewGroupRecord;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 流水线视图分组 Mapper 接口
 */
@Mapper
@Repository
public interface PipelineViewGroupDao {

    /**
     * 创建视图分组
     *
     * @param projectId 项目ID
     * @param viewId 视图ID
     * @param pipelineId 流水线ID
     * @param userId 用户ID
     */
    void create(
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId,
        @Param("pipelineId") String pipelineId,
        @Param("userId") String userId
    );

    /**
     * 获取视图分组列表（分页）
     *
     * @param viewId 视图ID
     * @param offset 偏移量
     * @param limit 每页数量
     * @return 视图分组记录列表
     */
    List<TPipelineViewGroupRecord> list(
        @Param("viewId") Long viewId,
        @Param("offset") Integer offset,
        @Param("limit") Integer limit
    );

    /**
     * 获取多个视图的分组列表
     *
     * @param projectId 项目ID
     * @param viewIds 视图ID列表
     * @return 视图分组记录列表
     */
    List<TPipelineViewGroupRecord> listByViewIds(
        @Param("projectId") String projectId,
        @Param("viewIds") List<Long> viewIds
    );

    /**
     * 获取指定视图的分组列表
     *
     * @param projectId 项目ID
     * @param viewId 视图ID
     * @return 视图分组记录列表
     */
    List<TPipelineViewGroupRecord> listByViewId(
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId
    );

    /**
     * 获取项目下的所有视图分组
     *
     * @param projectId 项目ID
     * @return 视图分组记录列表
     */
    List<TPipelineViewGroupRecord> listByProjectId(
        @Param("projectId") String projectId
    );

    /**
     * 获取流水线所属的视图分组
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @return 视图分组记录列表
     */
    List<TPipelineViewGroupRecord> listByPipelineId(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId
    );

    /**
     * 获取流水线所属的视图ID列表
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @return 视图ID列表
     */
    List<Long> listViewIdListByPipelineId(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId
    );

    /**
     * 获取视图包含的流水线ID列表
     *
     * @param projectId 项目ID
     * @param viewId 视图ID
     * @return 流水线ID列表
     */
    List<String> listPipelineIdByViewId(
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId
    );

    /**
     * 获取多个流水线所属的视图分组
     *
     * @param projectId 项目ID
     * @param pipelineIds 流水线ID集合
     * @return 视图分组记录列表
     */
    List<TPipelineViewGroupRecord> listByPipelineIds(
        @Param("projectId") String projectId,
        @Param("pipelineIds") Collection<String> pipelineIds
    );

    /**
     * 统计流水线所属的视图分组数量
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @return 数量
     */
    Integer countByPipelineId(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId
    );

    /**
     * 移除视图分组
     *
     * @param projectId 项目ID
     * @param viewId 视图ID
     * @param pipelineId 流水线ID
     */
    void remove(
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId,
        @Param("pipelineId") String pipelineId
    );

    /**
     * 移除视图的所有分组
     *
     * @param projectId 项目ID
     * @param viewId 视图ID
     */
    void remove(
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId
    );

    /**
     * 批量移除视图分组
     *
     * @param projectId 项目ID
     * @param viewId 视图ID
     * @param pipelineIds 流水线ID列表
     */
    void batchRemove(
        @Param("projectId") String projectId,
        @Param("viewId") Long viewId,
        @Param("pipelineIds") List<String> pipelineIds
    );

    /**
     * 统计多个视图的分组数量
     *
     * @param projectId 项目ID
     * @param viewIds 视图ID集合
     * @param filterPipelineIds 过滤的流水线ID列表（可选）
     * @return 视图ID到数量的映射
     */
    Map<Long, Integer> countByViewId(
        @Param("projectId") String projectId,
        @Param("viewIds") Collection<Long> viewIds,
        @Param("filterPipelineIds") List<String> filterPipelineIds
    );

    /**
     * 获取不重复的流水线ID列表
     *
     * @param projectId 项目ID
     * @param viewIds 视图ID集合
     * @return 流水线ID列表
     */
    List<String> distinctPipelineIds(
        @Param("projectId") String projectId,
        @Param("viewIds") Collection<Long> viewIds
    );

    /**
     * 删除流水线的视图分组
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @return 是否删除成功
     */
    boolean delete(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId
    );
} 