/*
 * Tencent is pleased to support the open source community by making BK-CI 蓝鲸持续集成平台 available.
 *
 * Copyright (C) 2019 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * BK-CI 蓝鲸持续集成平台 is licensed under the MIT license.
 *
 * A copy of the MIT License is included in this file.
 *
 *
 * Terms of the MIT License:
 * ---------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of
 * the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package cd.casic.ci.process.process.dal.pipeline;


import cd.casic.ci.common.pipeline.Model;
import cd.casic.ci.process.api.process.pojo.pipeline.PipelineResourceVersion;
import cd.casic.ci.process.api.process.pojo.setting.PipelineModelVersion;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

/**
 * 流水线资源数据访问接口
 */
@Mapper
@Repository
public interface PipelineResourceDao {

    /**
     * 创建流水线资源
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @param creator 创建者
     * @param version 版本号
     * @param versionName 版本名称
     * @param model 流水线模型
     * @param yamlStr YAML字符串
     * @param yamlVersion YAML版本
     * @param versionNum 版本序号
     * @param pipelineVersion 流水线版本
     * @param triggerVersion 触发器版本
     * @param settingVersion 设置版本
     */
    void create(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId,
        @Param("creator") String creator,
        @Param("version") int version,
        @Param("versionName") String versionName,
        @Param("model") Model model,
        @Param("yamlStr") String yamlStr,
        @Param("yamlVersion") String yamlVersion,
        @Param("versionNum") Integer versionNum,
        @Param("pipelineVersion") Integer pipelineVersion,
        @Param("triggerVersion") Integer triggerVersion,
        @Param("settingVersion") Integer settingVersion
    );

    /**
     * 更新发布版本
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @param version 版本号
     * @param versionName 版本名称
     * @param model 流水线模型
     * @param yamlStr YAML字符串
     * @param yamlVersion YAML版本
     * @param versionNum 版本序号
     * @param pipelineVersion 流水线版本
     * @param triggerVersion 触发器版本
     * @param settingVersion 设置版本
     */
    void updateReleaseVersion(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId,
        @Param("version") int version,
        @Param("versionName") String versionName,
        @Param("model") Model model,
        @Param("yamlStr") String yamlStr,
        @Param("yamlVersion") String yamlVersion,
        @Param("versionNum") Integer versionNum,
        @Param("pipelineVersion") Integer pipelineVersion,
        @Param("triggerVersion") Integer triggerVersion,
        @Param("settingVersion") Integer settingVersion
    );

    /**
     * 获取发布版本资源
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @return 流水线资源版本
     */
    PipelineResourceVersion getReleaseVersionResource(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId
    );

    /**
     * 获取最新版本模型字符串
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @return 模型字符串
     */
    String getLatestVersionModelString(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId
    );

    /**
     * 获取指定版本模型字符串
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @param version 版本号
     * @return 模型字符串
     */
    String getVersionModelString(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId,
        @Param("version") Integer version
    );

    /**
     * 获取最新模型资源列表
     *
     * @param pipelineIds 流水线ID集合
     * @param projectId 项目ID
     * @return 流水线ID、版本号、模型字符串
     */
    Map<String, Map<Integer, String>> listLatestModelResource(
        @Param("pipelineIds") Set<String> pipelineIds,
        @Param("projectId") String projectId
    );

    /**
     * 删除所有版本
     *
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @return 影响的行数
     */
    int deleteAllVersion(
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId
    );

    /**
     * 更新流水线模型
     *
     * @param userId 用户ID
     * @param pipelineModelVersion 流水线模型版本
     */
    void updatePipelineModel(
        @Param("userId") String userId,
        @Param("pipelineModelVersion") PipelineModelVersion pipelineModelVersion
    );

    /**
     * 更新设置版本
     *
     * @param userId 用户ID
     * @param projectId 项目ID
     * @param pipelineId 流水线ID
     * @param settingVersion 设置版本
     * @return 版本号
     */
    Integer updateSettingVersion(
        @Param("userId") String userId,
        @Param("projectId") String projectId,
        @Param("pipelineId") String pipelineId,
        @Param("settingVersion") int settingVersion
    );

    /**
     * 获取模型字符串列表
     *
     * @param projectId 项目ID
     * @param pipelineIds 流水线ID集合
     * @return 流水线ID到模型字符串的映射
     */
    Map<String, String> listModelString(
        @Param("projectId") String projectId,
        @Param("pipelineIds") Collection<String> pipelineIds
    );
} 