package cd.casic.ci.process.process.engine.control;


import cd.casic.ci.common.pipeline.Model;
import cd.casic.ci.common.pipeline.container.Container;
import cd.casic.ci.common.pipeline.container.Stage;
import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.common.pipeline.enums.CallBackEvent;


import cd.casic.ci.common.pipeline.enums.ProjectPipelineCallbackStatus;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.event.ProjectPipelineCallBack;


import cd.casic.ci.common.pipeline.service.utils.LogUtils;
import cd.casic.ci.process.api.engine.pojo.PipelineInfo;
import cd.casic.ci.process.api.process.pojo.ProjectPipelineCallBackHistory;
import cd.casic.ci.process.api.process.pojo.pipeline.ModelDetail;
import cd.casic.ci.process.process.constant.AllocIdGenerateConstant;
import cd.casic.ci.process.process.engine.pojo.event.PipelineStreamEnabledEvent;
import cd.casic.ci.process.process.service.base.PipelineBuildDetailServiceImpl;
import cd.casic.ci.process.process.service.base.engin.service.PipelineRepositoryService;
import cd.casic.ci.process.process.service.base.engin.service.ProjectPipelineCallBackService;
import cd.casic.ci.process.process.service.base.engin.service.ProjectPipelineCallBackUrlGenerator;
import cd.casic.ci.process.process.service.base.pojo.*;
import cd.casic.ci.process.process.service.base.pojo.constant.CallbackConstants;
import cd.casic.ci.project.service.ServiceAllocIdResource;
import cd.casic.framework.commons.util.watcher.Watcher;


import io.github.resilience4j.circuitbreaker.CallNotPermittedException;
import io.github.resilience4j.circuitbreaker.CircuitBreaker;
import io.github.resilience4j.circuitbreaker.CircuitBreakerRegistry;
import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.Tags;
import okhttp3.*;
import org.apache.commons.lang3.tuple.ImmutableTriple;
import org.apache.commons.lang3.tuple.Triple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.TimeUnit;


@Service
public class CallBackControl {
    private static final Logger logger = LoggerFactory.getLogger(CallBackControl.class);
    private static final MediaType JSON = MediaType.parse("application/json;charset=utf-8");
    private static final int MAX_RETRY_COUNT = 3;
    private static final long DEFAULT_FAILURE_DISABLE_TIME_PERIOD = 12 * 60 * 60 * 1000L;
    private static final String PIPELINE_CALLBACK_COUNT = "pipeline_callback_count";
    private static final long connectTimeout = 3L;
    private static final long readTimeout = 3L;
    private static final long writeTimeout = 3L;

    private final PipelineBuildDetailServiceImpl pipelineBuildDetailService;
    private final PipelineRepositoryService pipelineRepositoryService;
    private final ProjectPipelineCallBackService projectPipelineCallBackService;
    private final CircuitBreakerRegistry callbackCircuitBreakerRegistry;
    private final MeterRegistry meterRegistry;
    private final ProjectPipelineCallBackUrlGenerator projectPipelineCallBackUrlGenerator;
    private ServiceAllocIdResource serviceAllocIdResource;

    @Value("${callback.failureDisableTimePeriod:#{43200000}}")
    private long failureDisableTimePeriod = DEFAULT_FAILURE_DISABLE_TIME_PERIOD;

    @Autowired
    public CallBackControl(
            PipelineBuildDetailServiceImpl pipelineBuildDetailService,
            PipelineRepositoryService pipelineRepositoryService,
            ProjectPipelineCallBackService projectPipelineCallBackService,
            CircuitBreakerRegistry callbackCircuitBreakerRegistry,
            MeterRegistry meterRegistry,
            ProjectPipelineCallBackUrlGenerator projectPipelineCallBackUrlGenerator,
            ServiceAllocIdResource serviceAllocIdResource
    ) {
        this.pipelineBuildDetailService = pipelineBuildDetailService;
        this.pipelineRepositoryService = pipelineRepositoryService;
        this.projectPipelineCallBackService = projectPipelineCallBackService;
        this.callbackCircuitBreakerRegistry = callbackCircuitBreakerRegistry;
        this.meterRegistry = meterRegistry;
        this.projectPipelineCallBackUrlGenerator = projectPipelineCallBackUrlGenerator;
        this.serviceAllocIdResource = serviceAllocIdResource;
    }

    // 静态初始化块
    private static final SSLSocketFactory anySslSocketFactory;
    private static final TrustManager[] trustAnyCerts;
    private static final OkHttpClient callbackClient;

    static {
        trustAnyCerts = new TrustManager[]{
                new X509TrustManager() {
                    @Override
                    public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                    }

                    @Override
                    public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                    }

                    @Override
                    public X509Certificate[] getAcceptedIssuers() {
                        return new X509Certificate[0];
                    }
                }
        };

        try {
            SSLContext sslContext = SSLContext.getInstance("SSL");
            sslContext.init(null, trustAnyCerts, new java.security.SecureRandom());
            anySslSocketFactory = sslContext.getSocketFactory();
        } catch (Exception e) {
//            throw new RemoteServiceException(e.getMessage()); TODO 添加项目报错
            throw new RuntimeException(e.getMessage());
        }

        callbackClient = new OkHttpClient.Builder()
                .connectTimeout(connectTimeout, TimeUnit.SECONDS)
                .readTimeout(readTimeout, TimeUnit.SECONDS)
                .writeTimeout(writeTimeout, TimeUnit.SECONDS)
                .sslSocketFactory(anySslSocketFactory, (X509TrustManager) trustAnyCerts[0])
                .hostnameVerifier((hostname, session) -> true)
                .build();
    }

    public void pipelineCreateEvent(String projectId, String pipelineId) {
        callBackPipelineEvent(projectId, pipelineId, CallBackEvent.CREATE_PIPELINE);
    }

    public void pipelineDeleteEvent(String projectId, String pipelineId) {
        callBackPipelineEvent(projectId, pipelineId, CallBackEvent.DELETE_PIPELINE);
    }

    public void pipelineUpdateEvent(String projectId, String pipelineId) {
        callBackPipelineEvent(projectId, pipelineId, CallBackEvent.UPDATE_PIPELINE);
    }

    public void pipelineRestoreEvent(String projectId, String pipelineId) {
        callBackPipelineEvent(projectId, pipelineId, CallBackEvent.RESTORE_PIPELINE);
    }

    public void projectCreate(String projectId, String projectName, String userId) {
        callBackProjectEvent(projectId, projectName, userId, true, CallBackEvent.PROJECT_CREATE);
    }

    public void projectUpdate(String projectId, String projectName, String userId) {
        callBackProjectEvent(projectId, projectName, userId, true, CallBackEvent.PROJECT_UPDATE);
    }

    public void projectEnable(String projectId, String projectName, String userId) {
        callBackProjectEvent(projectId, projectName, userId, true, CallBackEvent.PROJECT_ENABLE);
    }

    public void projectDisable(String projectId, String projectName, String userId) {
        callBackProjectEvent(projectId, projectName, userId, false, CallBackEvent.PROJECT_DISABLE);
    }

    public void pipelineStreamEnabledEvent(PipelineStreamEnabledEvent event) {
        logger.info(event.getProjectId() + "|STREAM_ENABLED|callback stream enable event");
        List<ProjectPipelineCallBack> list = projectPipelineCallBackService.listProjectCallBack(
                event.getProjectId(),
                CallBackEvent.STREAM_ENABLED.name()
        );
        StreamEnabledEvent streamEnabledEvent = new StreamEnabledEvent(
                event.getUserId(),
                event.getGitProjectId(),
                event.getGitProjectUrl(),
                event.getEnable()
        );
        sendToCallBack(new CallBackData<StreamEnabledEvent>(CallBackEvent.STREAM_ENABLED, streamEnabledEvent), list);
    }

    private void callBackPipelineEvent(String projectId, String pipelineId, CallBackEvent callBackEvent) {
        logger.info(projectId + "|" + pipelineId + "|" + callBackEvent + "|callback pipeline event");
        List<ProjectPipelineCallBack> list = projectPipelineCallBackService.listProjectCallBack(
                projectId,
                callBackEvent.name()
        );
        if (list.isEmpty()) {
            logger.info("[" + pipelineId + "]|[" + callBackEvent + "]| no callback");
            return;
        }

        PipelineInfo pipelineInfo = pipelineRepositoryService.getPipelineInfo(
                projectId,
                pipelineId,
                null,
                false
        );
        if (pipelineInfo == null) {
            return;
        }

        PipelineEvent pipelineEvent = new PipelineEvent(
                pipelineInfo.getPipelineId(),
                pipelineInfo.getPipelineName(),
                pipelineInfo.getLastModifyUser(),
                pipelineInfo.getUpdateTime(),
                pipelineInfo.getProjectId()
        );

        sendToCallBack(new CallBackData<>(callBackEvent, pipelineEvent), list);
    }

    private void callBackProjectEvent(
            String projectId,
            String projectName,
            String userId,
            boolean enable,
            CallBackEvent callBackEvent
    )  {
        logger.info(projectId + "|" + projectName + "|" + callBackEvent + "|callback project event");
        List<ProjectPipelineCallBack> list = projectPipelineCallBackService.listProjectCallBack(
                CallbackConstants.DEVOPS_ALL_PROJECT,
                callBackEvent.name()
        );
        if (list.isEmpty()) {
            logger.info("no [" + callBackEvent + "] project callback");
            return;
        }

        ProjectCallbackEvent projectEvent = new ProjectCallbackEvent(
                projectId,
                projectName,
                enable,
                userId
        );

        sendToCallBack(new CallBackData<>(callBackEvent, projectEvent), list);
    }

    public void callBackBuildEvent(PipelineBuildStatusBroadCastEvent event) {
        String projectId = event.getProjectId();
        String pipelineId = event.getPipelineId();
        String buildId = event.getBuildId();
        if (event.getAtomCode() != null && VmOperateTaskGenerator.isVmAtom(event.getAtomCode())) {
            return;
        }
        CallBackEvent callBackEvent = EventUtils.toEventType(event);
        if (callBackEvent == null) {
            return;
        }

        logger.info(projectId + "|" + pipelineId + "|" + buildId + "|" + callBackEvent.name() + "|" +
                event.getStageId() + "|" + event.getTaskId() + "|callback");

        List<ProjectPipelineCallBack> list = new ArrayList<>();
        list.addAll(projectPipelineCallBackService.listProjectCallBack(
                projectId,
                callBackEvent.name()
        ));

        List<ProjectPipelineCallBack> pipelineCallback = projectPipelineCallBackService.getPipelineCallback(
                projectId,
                pipelineId,
                callBackEvent.name()
        );
        if (pipelineCallback.isEmpty()) {
            pipelineCallback = pipelineRepositoryService.getPipelineResourceVersion(
                    projectId,
                    pipelineId,
                    null,
                    false
            ).getModel().getPipelineCallBack(
                    projectId,
                    callBackEvent
            );
            if (pipelineCallback == null) {
                pipelineCallback = Collections.emptyList();
            }
        }
        if (!pipelineCallback.isEmpty()) {
            list.addAll(pipelineCallback);
        }

        if (list.isEmpty()) {
            return;
        }

        ModelDetail modelDetail = pipelineBuildDetailService.get(
                projectId,
                event.getBuildId(),
                false
        );
        if (modelDetail == null) {
            return;
        }

        BuildEvent buildEvent = new BuildEvent(
                event.getBuildId(),
                modelDetail.getPipelineId(),
                modelDetail.getPipelineName(),
                modelDetail.getUserId(),
                modelDetail.getTriggerUser(),
                modelDetail.getCancelUserId(),
                modelDetail.getStatus(),
                modelDetail.getStartTime(),
                modelDetail.getEndTime() != null ? modelDetail.getEndTime() : 0,
                new SimpleModel(parseModel(modelDetail.getModel())),
                event.getProjectId(),
                modelDetail.getTrigger(),
                event.getStageId(),
                event.getTaskId(),
                modelDetail.getBuildNum(),
                modelDetail.getDebug()
        );
        sendToCallBack(new CallBackData<>(callBackEvent, buildEvent), list);
    }

    private <T> void sendToCallBack(CallBackData<T> callBackData, List<ProjectPipelineCallBack> list){
        String requestBody = com.alibaba.fastjson.JSON.toJSONString(callBackData);

        list.forEach(callBack -> {
            String uniqueId = "";
            if (callBackData.getData() instanceof PipelineEvent) {
                uniqueId = ((PipelineEvent) callBackData.getData()).getPipelineId();
            } else if (callBackData.getData() instanceof BuildEvent) {
                uniqueId = ((BuildEvent) callBackData.getData()).getBuildId();
            }

            Watcher watcher = new Watcher(callBack.getProjectId() + "|" + callBack.getCallBackUrl() +
                    "|" + callBack.getEvents() + "|" + uniqueId);
            try {
                logger.info(callBack.getProjectId() + "|" + callBack.getCallBackUrl() + "|" +
                        uniqueId + "|" + callBack.getEvents() + "|send to callback");
                if (callBack.getCallBackUrl().isBlank()) {
                    logger.warn("[" + callBack.getProjectId() + "]| call back url is empty!");
                    return;
                }
                send(uniqueId, callBack, requestBody);
            } catch (Exception e) {
                logger.error("BKSystemErrorMonitor|" + callBack.getProjectId() + "|" +
                        callBack.getCallBackUrl() + "|" + callBack.getEvents() + "|" + e.getMessage(), e);
            } finally {
                watcher.stop();
                LogUtils.printCostTimeWE(watcher, 2000,5000);
            }
        });
    }

    private void send(String uniqueId, ProjectPipelineCallBack callBack, String requestBody) {
        long startTime = System.currentTimeMillis();
        Request.Builder builder = new Request.Builder()
                .url(callBack.getCallBackUrl())
                .header("X-DEVOPS-WEBHOOK-TOKEN", callBack.getSecretToken() != null ? callBack.getSecretToken() : "NONE")
//  TODO              .header(TraceTag.TRACE_HEADER_DEVOPS_BIZID, TraceTag.buildBiz())
                .post(RequestBody.create(JSON, requestBody));

        if (callBack.getSecretParam() != null) {
            callBack.getSecretParam().secret(builder);
        }

        Request request = builder.build();

        String errorMsg = null;
        ProjectPipelineCallbackStatus status = ProjectPipelineCallbackStatus.SUCCESS;
        CircuitBreaker breaker = callbackCircuitBreakerRegistry.circuitBreaker(callBack.getCallBackUrl());
        try {
            breaker.executeCallable(() -> {
//          TODO      HttpRetryUtils.retry(MAX_RETRY_COUNT, () -> {
//                    callbackClient.newCall(request).execute().closeQuietly();
//                });
                if (callBack.getFailureTime() != null) {
                    projectPipelineCallBackService.updateFailureTime(
                            callBack.getProjectId(),
                            callBack.getId(),
                            null
                    );
                }
                return null;
            });
        } catch (CallNotPermittedException e) {
            logger.warn(
                    "[" + callBack.getProjectId() + "]|CALL_BACK|url=" + callBack.getCallBackUrl() +
                            "|" + callBack.getEvents() + "|failureRate=" + breaker.getMetrics().getFailureRate() +
                            "|" + e.getMessage()
            );
            checkIfNeedDisable(breaker, callBack);
            errorMsg = e.getMessage();
            status = ProjectPipelineCallbackStatus.FAILED;
        } catch (Exception e) {
            logger.warn(
                    "BKSystemErrorMonitor|[" + callBack.getProjectId() + "]|CALL_BACK|" +
                            "url=" + callBack.getCallBackUrl() + "|" + callBack.getEvents(),
                    e
            );
            errorMsg = e.getMessage();
            status = ProjectPipelineCallbackStatus.FAILED;
        } finally {
            String originUrl = projectPipelineCallBackUrlGenerator.decodeCallbackUrl(
                    request.url().toString()
            );
            String realUrl = originUrl.substring(originUrl.indexOf("?"));
            try {
                String host = new URL(realUrl).getHost();
                Counter.builder(PIPELINE_CALLBACK_COUNT)
                        .tags(
                                Tags.of("status", status.name())
                                        .and("host",host)
                                        .and("event", callBack.getEvents())
                        )
                        .register(meterRegistry)
                        .increment();
            } catch (MalformedURLException e) {
                throw new RuntimeException(e);
            }

            saveHistory(
                    callBack,
                    Collections.singletonList(new ProjectPipelineCallBackHistory.CallBackHeader("X-DEVOPS-WEBHOOK-UNIQUE-ID", uniqueId)),
                    status.name(),
                    errorMsg,
                    startTime,
                    System.currentTimeMillis()
            );
        }
    }

    private void checkIfNeedDisable(CircuitBreaker breaker, ProjectPipelineCallBack callBack) {
        if (breaker.getMetrics().getFailureRate() == 100.0F) {
            long duration = 0;
            if (callBack.getFailureTime() != null) {
                duration = System.currentTimeMillis() - callBack.getFailureTime().atOffset(ZoneOffset.UTC).toInstant().toEpochMilli();
            } else {
                projectPipelineCallBackService.updateFailureTime(
                        callBack.getProjectId(),
                        callBack.getId(),
                        LocalDateTime.now()
                );
            }
            if (duration >= failureDisableTimePeriod) {
                logger.warn(
                        "disable callbacks because of 100% failure rate|" +
                                "[" + callBack.getProjectId() + "]|CALL_BACK|url=" + callBack.getCallBackUrl() +
                                "|" + callBack.getEvents() + "|duration=" + duration
                );
                projectPipelineCallBackService.disable(callBack.getProjectId(), callBack.getId());
            }
        }
    }

    private void saveHistory(
            ProjectPipelineCallBack callBack,
            List<ProjectPipelineCallBackHistory.CallBackHeader> requestHeaders,
            String status,
            String errorMsg,
            long startTime,
            long endTime
    ) {
        try {
            projectPipelineCallBackService.createHistory(
                    new ProjectPipelineCallBackHistory(
                            serviceAllocIdResource.generateSegmentId(AllocIdGenerateConstant.PROJECT_PIPELINE_CALLBACK_HISTORY)
                            ,
                            callBack.getProjectId(),
                            callBack.getCallBackUrl(),
                            callBack.getEvents(),
                            status,
                            requestHeaders,
                            "",
                            0,
                            "",
                            errorMsg,
                            null,
                            startTime,
                            endTime
//                            client.get(ServiceAllocIdResource.class)
//                                    .generateSegmentId("PROJECT_PIPELINE_CALLBACK_HISTORY").getData()
                    )
            );
        } catch (Throwable e) {
            logger.error("[" + callBack.getProjectId() + "]|[" + callBack.getCallBackUrl() +
                    "]|[" + callBack.getEvents() + "]|save fail", e);
        }
    }

    List<SimpleStage> parseModel(Model model) {
        List<SimpleStage> stages = new ArrayList<>();
        for (int pos = 0; pos < model.getStages().size(); pos++) {
            Stage s = model.getStages().get(pos);
            List<SimpleJob> jobs = new ArrayList<>();
            SimpleStage stage = new SimpleStage(
                    "Stage-" + (pos + 1),
                    s.getName() != null ? s.getName() : "",
                    jobs
            );
            logger.info("parseModel " + model.getName() + "|" + stage.getStageName() +
                    "|" + stage.getName() + "|");
            stage.setStartTime(s.getStartEpoch() != null ? s.getStartEpoch() : 0);
            stages.add(stage);

            Triple<Long, Long, BuildStatus> triple = parseJob(s, jobs);
            long stageStartTimeMills = triple.getLeft();
            long stageEndTimeMills = triple.getMiddle();
            BuildStatus stageStatus = triple.getRight();

            if (stage.getStartTime() == 0L && stageStartTimeMills > 0) {
                stage.setStartTime(stageStartTimeMills);
            }

            if (stageEndTimeMills > 0) {
                stage.setEndTime(stageEndTimeMills);
            }

            stage.setStatus(stageStatus.name());
        }
        return stages;
    }

    private Triple<Long, Long, BuildStatus> parseJob(Stage s, List<SimpleJob> jobs) {
        long stageStartTimeMills = Long.MAX_VALUE;
        long stageEndTimeMills = 1L;
        BuildStatus stageStatus = BuildStatus.QUEUE;
        for (Container c : s.getContainers()) {
            BuildStatus jobStatus = BuildStatus.parse(c.getStatus());
            long jobStartTimeMills = c.getStartEpoch() != null ? c.getStartEpoch() : 0L;
            long jobEndTimeMills = jobStatus.isFinish() ?
                    jobStartTimeMills + (c.getElementElapsed() != null ? c.getElementElapsed() : 0) +
                            (c.getSystemElapsed() != null ? c.getSystemElapsed() : 0) : 0;

            if (stageStartTimeMills > jobStartTimeMills) {
                stageStartTimeMills = jobStartTimeMills;
            }

            if (jobEndTimeMills == 0L) {
                stageStatus = jobStatus;
                stageEndTimeMills = jobEndTimeMills;
            } else if (stageEndTimeMills >= 1 && stageEndTimeMills < jobEndTimeMills) {
                stageEndTimeMills = jobEndTimeMills;
                if (jobStatus.isFailure()) {
                    stageStatus = jobStatus;
                }
            }

            jobs.add(
                    new SimpleJob(
                            c.getName(),
                            jobStatus.name(),
                            jobStartTimeMills,
                            jobEndTimeMills,
                            parseTask(c)
                    )
            );
        }

        if (stageEndTimeMills > 0 && !stageStatus.isFinish()) {
            stageStatus = BuildStatus.SUCCEED;
        }
        return new ImmutableTriple<>(stageStartTimeMills, stageEndTimeMills, stageStatus);
    }

    private List<SimpleTask> parseTask(Container c) {
        List<SimpleTask> tasks = new ArrayList<>();
        for (Element e : c.getElements()) {
            long taskStartTimeMills = e.getStartEpoch() != null ? e.getStartEpoch() : 0;
            BuildStatus taskStatus = BuildStatus.parse(e.getStatus());
            long taskEndTimeMills = taskStatus.isFinish() ?
                    taskStartTimeMills + (e.getElapsed() != null ? e.getElapsed() : 0) : 0;
            tasks.add(
                    new SimpleTask(
                            e.getId(),
                            e.getName(),
                            e.getAtomCode(),
                            taskStatus.name(),
                            taskStartTimeMills,
                            taskEndTimeMills
                    )
            );
        }
        return tasks;
    }
}
