package cd.casic.ci.process.process.engine.service;

import cd.casic.ci.common.pipeline.Model;
import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.common.pipeline.enums.ChannelCode;
import cd.casic.ci.common.pipeline.enums.StartType;
import cd.casic.ci.common.pipeline.pojo.BuildParameters;
import cd.casic.ci.common.pipeline.pojo.ErrorInfo;
import cd.casic.ci.common.pipeline.pojo.time.BuildRecordTimeCost;
import cd.casic.ci.process.api.engine.pojo.BuildInfo;
import cd.casic.ci.process.api.process.enums.HistorySearchType;
import cd.casic.ci.process.api.process.pojo.*;
import cd.casic.ci.process.api.process.pojo.app.StartBuildContext;
import cd.casic.ci.process.api.process.pojo.pipeline.PipelineLatestBuild;
import cd.casic.ci.process.process.dataObject.TPipelineBuildSummaryRecord;
import cd.casic.ci.process.process.dataObject.TPipelineInfoRecord;
import cd.casic.ci.process.process.pojo.LatestRunningBuild;
import cd.casic.ci.process.process.pojo.PipelineFilterParam;
import org.apache.commons.lang3.tuple.Pair;

import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 流水线运行时相关的服务接口
 */
public interface PipelineRuntimeService {

    /**
     * 取消待处理的任务
     */
    void cancelPendingTask(String projectId, String pipelineId, String userId);

    /**
     * 获取构建信息
     */
    BuildInfo getBuildInfo(String projectId, String buildId);

    /**
     * 获取构建信息
     */
    BuildInfo getBuildInfo(String projectId, String pipelineId, String buildId);

    /**
     * 获取运行中的构建数量
     */
    int getRunningBuildCount(String projectId, String pipelineId);

    /**
     * 根据并发组获取构建列表
     */
    List<Pair<String, String>> getBuildInfoListByConcurrencyGroup(
            String projectId,
            String concurrencyGroup,
            List<BuildStatus> status
    );

    /**
     * 获取构建号
     */
    Map<String, String> getBuildNoByByPair(Set<String> buildIds, String projectId);

    /**
     * 获取构建摘要记录
     */
    TPipelineBuildSummaryRecord getBuildSummaryRecord(String projectId, String pipelineId);

    /**
     * 获取流水线构建记录
     */
   TPipelineInfoRecord getBuildPipelineRecords(
            String projectId,
            ChannelCode channelCode,
            Collection<String> pipelineIds,
            PipelineSortType sortType,
            List<String> favorPipelines,
            List<String> authPipelines,
            String viewId,
            List<PipelineFilterParam> pipelineFilterParamList,
            Boolean permissionFlag,
            Integer page,
            Integer pageSize
    );

    /**
     * 获取最新构建
     */
    Map<String, PipelineLatestBuild> getLatestBuild(String projectId, List<String> pipelineIds);

    /**
     * 获取流水线构建历史
     */
    List<BuildHistory> listPipelineBuildHistory(
            String projectId,
            String pipelineId,
            int offset,
            int limit,
            Boolean updateTimeDesc
    );

    /**
     * 获取流水线构建历史
     */
    List<BuildHistory> listPipelineBuildHistory(
            String projectId,
            String pipelineId,
            int offset,
            int limit,
            List<String> materialAlias,
            String materialUrl,
            List<String> materialBranch,
            String materialCommitId,
            String materialCommitMessage,
            List<BuildStatus> status,
            List<StartType> trigger,
            Long queueTimeStartTime,
            Long queueTimeEndTime,
            Long startTimeStartTime,
            Long startTimeEndTime,
            Long endTimeStartTime,
            Long endTimeEndTime,
            Long totalTimeMin,
            Long totalTimeMax,
            String remark,
            Integer buildNoStart,
            Integer buildNoEnd,
            String buildMsg,
            List<String> startUser,
            Boolean updateTimeDesc,
            Boolean debug,
            List<String> triggerAlias,
            List<String> triggerBranch,
            List<String> triggerUser
    );

    /**
     * 更新构建备注
     */
    void updateBuildRemark(String projectId, String pipelineId, String buildId, String remark);

    /**
     * 获取历史条件仓库
     */
    List<String> getHistoryConditionRepo(
            String projectId,
            String pipelineId,
            Integer debugVersion,
            String search,
            HistorySearchType type
    );

    /**
     * 获取历史条件分支
     */
    List<String> getHistoryConditionBranch(
            String projectId,
            String pipelineId,
            List<String> aliasList,
            Integer debugVersion,
            String search,
            HistorySearchType type
    );

    /**
     * 获取构建历史
     */
    BuildHistory getBuildHistoryByBuildNum(
            String projectId,
            String pipelineId,
            Integer buildNum,
            Set<BuildStatus> statusSet,
            Boolean debug
    );

    /**
     * 获取构建基本信息
     */
    Map<String, BuildBasicInfo> getBuildBasicInfoByIds(Set<String> buildIds);

    /**
     * 获取构建历史
     */
    BuildHistory getBuildHistoryById(String projectId, String buildId);

    /**
     * 获取启动用户
     */
    String getStartUser(String projectId, String buildId);

    /**
     * 获取构建历史
     */
    List<BuildHistory> getBuildHistoryByIds(
            Set<String> buildIds,
            String startBeginTime,
            String endBeginTime,
            String projectId
    );

    /**
     * 取消构建
     */
    Boolean cancelBuild(
            String projectId,
            String pipelineId,
            String buildId,
            String userId,
            int executeCount,
            BuildStatus buildStatus,
            Boolean terminateFlag
    );

    /**
     * 开始构建
     */
    BuildId startBuild(Model fullModel, StartBuildContext context);

    /**
     * 更新构建号
     */
    void updateBuildNo(String projectId, String pipelineId, int buildNo, Boolean debug);

    /**
     * 更新执行次数
     */
    void updateExecuteCount(String projectId, String buildId, int executeCount);

    /**
     * 开始最新运行构建
     */
    void startLatestRunningBuild(LatestRunningBuild latestRunningBuild);

    /**
     * 完成最新运行构建
     */
    void finishLatestRunningBuild(
            LatestRunningBuild latestRunningBuild,
            BuildStatus currentBuildStatus,
            List<ErrorInfo> errorInfoList,
            BuildRecordTimeCost timeCost
    );

    /**
     * 获取构建参数
     */
    List<BuildParameters> getBuildParametersFromStartup(
            String projectId,
            String buildId
    );

    /**
     * 获取执行时间
     */
    Long getExecuteTime(String projectId, String buildId);

    /**
     * 获取上次构建
     */
    BuildInfo getLastTimeBuild(String projectId, String pipelineId, Boolean debug);

    /**
     * 获取流水线构建历史数量
     */
    int getPipelineBuildHistoryCount(String projectId, String pipelineId, Integer debugVersion);

    /**
     * 获取构建
     */
    List<String> getBuilds(
            String projectId,
            String pipelineId,
            Set<BuildStatus> buildStatus,
            Integer debugVersion
    );

    /**
     * 获取流水线构建历史数量
     */
    int getPipelineBuildHistoryCount(
            String projectId,
            String pipelineId,
            List<String> materialAlias,
            String materialUrl,
            List<String> materialBranch,
            String materialCommitId,
            String materialCommitMessage,
            List<BuildStatus> status,
            List<StartType> trigger,
            Long queueTimeStartTime,
            Long queueTimeEndTime,
            Long startTimeStartTime,
            Long startTimeEndTime,
            Long endTimeStartTime,
            Long endTimeEndTime,
            Long totalTimeMin,
            Long totalTimeMax,
            String remark,
            Integer buildNoStart,
            Integer buildNoEnd,
            String buildMsg,
            List<String> startUser,
            Boolean debug,
            List<String> triggerAlias,
            List<String> triggerBranch,
            List<String> triggerUser
    );

    /**
     * 获取总构建历史数量
     */
    int getTotalBuildHistoryCount(
            String projectId,
            String pipelineId,
            List<BuildStatus> status,
            Long startTimeEndTime
    );

    /**
     * 获取所有构建号
     */
    Collection<Integer> getAllBuildNum(String projectId, String pipelineId, Integer debugVersion);

    /**
     * 获取最新构建ID
     */
    String getLatestBuildId(String projectId, String pipelineId);

    /**
     * 获取最新完成构建ID
     */
    String getLatestFinishedBuildId(String projectId, String pipelineId);

    /**
     * 获取最新成功构建ID
     */
    String getLatestSucceededBuildId(String projectId, String pipelineId);

    /**
     * 获取最新失败构建ID
     */
    String getLatestFailedBuildId(String projectId, String pipelineId);

    /**
     * 根据构建号获取构建ID
     */
    String getBuildIdByBuildNum(
            String projectId,
            String pipelineId,
            int buildNum,
            Integer debugVersion
    );

    /**
     * 更新构建信息状态为排队
     */
    void updateBuildInfoStatus2Queue(String projectId, String buildId, BuildStatus oldStatus, String showMsg);

    /**
     * 更新制品列表
     */
    Boolean updateArtifactList(
            String projectId,
            String pipelineId,
            String buildId,
            String artifactListJsonString
    );

    /**
     * 更新构建历史阶段状态
     */
    void updateBuildHistoryStageState(String projectId, String buildId, List<BuildStageStatus> allStageStatus);

    /**
     * 更新推荐版本
     */
    void updateRecommendVersion(String projectId, String buildId, String recommendVersion);

    /**
     * 更新构建参数
     */
    Boolean updateBuildParameters(
            String projectId,
            String pipelineId,
            String buildId,
            Collection<BuildParameters> buildParameters,
            Boolean debug
    );

    /**
     * 并发取消构建流水线
     */
    void concurrencyCancelBuildPipeline(
            String projectId,
            String pipelineId,
            String buildId,
            String userId,
            String groupName,
            String detailUrl
    );

    /**
     * 更新异步状态
     */
    void updateAsyncStatus(
            String projectId,
            String pipelineId,
            String buildId,
            String taskId,
            int executeCount,
            String asyncStatus
    );

    /**
     * 获取构建变量服务
     */
    Map<String, String> getBuildVariableService(
            String projectId,
            String pipelineId,
            String buildId,
            Set<String> keys
    );
}
