package cd.casic.ci.process.process.service.base;


import cd.casic.ci.common.pipeline.Model;
import cd.casic.ci.common.pipeline.container.Container;
import cd.casic.ci.common.pipeline.container.Stage;
import cd.casic.ci.process.process.service.base.dispatcher.PipelineEventDispatcher;
import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.common.pipeline.enums.ChannelCode;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.process.api.process.pojo.BuildStageStatus;
import cd.casic.ci.process.process.dal.pipeline.PipelineBuildDao;
import cd.casic.ci.process.process.dal.record.BuildDetailDao;
import cd.casic.ci.process.process.dataObject.TPipelineBuildDetailRecord;
import cd.casic.ci.process.process.service.base.pojo.PipelineBuildWebSocketPushEvent;
import cd.casic.ci.process.process.service.base.pojo.enums.RefreshType;
import cd.casic.framework.commons.util.watcher.LogUtils;
import cd.casic.framework.commons.util.watcher.Watcher;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.assertj.core.util.Preconditions;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;

import static cd.casic.ci.common.pipeline.constant.CommonConstants.*;

@Slf4j
@Service
public class BaseBuildDetailServiceImpl {


    protected static final String TRIGGER_STAGE = "stage-1";

    protected  PipelineBuildDao pipelineBuildDao;
    protected  BuildDetailDao buildDetailDao;
    private  StageTagServiceImpl stageTagServiceImpl;
    private PipelineEventDispatcher pipelineEventDispatcher;

    // TODO 更改redis逻辑
//    protected  RedisOperation redisOperation;

    public BaseBuildDetailServiceImpl(
            PipelineBuildDao pipelineBuildDao,
            BuildDetailDao buildDetailDao,
            StageTagServiceImpl stageTagServiceImpl,
            PipelineEventDispatcher pipelineEventDispatcher
//            RedisOperation redisOperation
    ) {

        this.pipelineBuildDao = pipelineBuildDao;
        this.buildDetailDao = buildDetailDao;
        this.stageTagServiceImpl = stageTagServiceImpl;
        this.pipelineEventDispatcher = pipelineEventDispatcher;
//        this.redisOperation = redisOperation;
    }

    public Model getBuildModel(String projectId, String buildId) {
        var record = buildDetailDao.get(projectId, buildId);
        if (record == null) {
            return null;
        }
        return JSON.parseObject(record.getModel(), Model.class);
    }

    protected Model update(
            String projectId,
            String buildId,
            ModelInterface modelInterface,
            BuildStatus buildStatus,
            String cancelUser,
            String operation
    ) {
        Watcher watcher = new Watcher("updateDetail#" + buildId + "#" + operation);
        String message = "nothing";
        TPipelineBuildDetailRecord record = null;
//        var lock = new PipelineBuildDetailLock(redisOperation, buildId);
        ReentrantLock lock = new ReentrantLock();
        try {
            watcher.start("lock");
            lock.lock();

            watcher.start("getDetail");
            record = buildDetailDao.get(projectId, buildId);
            Preconditions.checkArgument(record != null, "The build detail is not exist");

            watcher.start("model");
            var model = JSON.parseObject(record.getModel(), Model.class);

            watcher.start("traverseModel");
            traverseModel(model, modelInterface);
            watcher.stop();

            String modelStr = null;
            if (modelInterface.needUpdate()) {
                watcher.start("toJson");
                modelStr = JSON.toJSONString(model, false);
            }

            var statusPair = takeBuildStatus(record, buildStatus);
            if ((modelStr == null || modelStr.isBlank()) && !statusPair.getFirst()) {
                message = "Will not update";
                return model;
            }

            watcher.start("updateModel");
            buildDetailDao.update(
                    projectId,
                    buildId,
                    modelStr,
                    statusPair.getSecond(),
                    cancelUser != null ? cancelUser :
                            (buildStatus.isCancel() && record.getCancelUser() == null) ? "System" : null
            );

            message = "update done";
            return model;
        } catch (Throwable ignored) {
            message = ignored.getMessage() != null ? ignored.getMessage() : "";
            log.warn("[" + buildId + "]| Fail to update the build detail: " + ignored.getMessage(), ignored);
            watcher.start("getDetail");
            Preconditions.checkArgument(record != null, "The build detail is not exist");
            watcher.start("model");
            return JSON.parseObject(record.getModel(), Model.class);
        } finally {
            lock.unlock();
            log.info("[" + buildId + "|" + buildStatus + "]|" + operation + "|update_detail_model| " + message);
            if (message.equals("update done")) {
                watcher.start("dispatchEvent");
                pipelineDetailChangeEvent(projectId, buildId);
            }
            LogUtils.printCostTimeWE(watcher);
        }
    }

    protected List<BuildStageStatus> fetchHistoryStageStatus(
            Model model,
            BuildStatus buildStatus,
            List<String> reviewers,
            String errorMsg,
            String cancelUser
    ) {
        Map<String, String> stageTagMap = stageTagServiceImpl.getAllStageTag().stream()
                .collect(Collectors.toMap(tag -> tag.getId(), tag -> tag.getStageTagName()));

        String statusMessage;
        String reason;
        if (buildStatus == BuildStatus.REVIEWING) {
            statusMessage = BUILD_REVIEWING;
            reason = reviewers != null ? String.join(",", reviewers) : null;
        } else if (buildStatus == BuildStatus.STAGE_SUCCESS) {
            statusMessage = BUILD_STAGE_SUCCESS;
            reason = null;
        } else if (buildStatus.isFailure()) {
            statusMessage = BUILD_FAILED;
            reason = errorMsg != null ? errorMsg : buildStatus.name();
        } else if (buildStatus.isCancel()) {
            statusMessage = BUILD_CANCELED;
            reason = cancelUser;
        } else if (buildStatus.isSuccess()) {
            statusMessage = BUILD_COMPLETED;
            reason = null;
        } else {
            statusMessage = BUILD_RUNNING;
            reason = null;
        }

        return model.getStages().stream()
                .map(stage -> new BuildStageStatus(
                        stage.getId(),
                        stage.getName() != null ? stage.getName() : stage.getId(),
                        stage.getStatus(),
                        stage.getTag() != null ? stage.getTag().stream()
                                .map(tag -> stageTagMap.getOrDefault(tag, "null"))
                                .collect(Collectors.toList()) : null,
                        stage.getStartEpoch(),
                        stage.getElapsed(),
                        null,
                        stage.getId().equals(TRIGGER_STAGE) ?
                                "" : null
                ))
                .collect(Collectors.toList());
    }

    private Pair<Boolean, BuildStatus> takeBuildStatus(
            TPipelineBuildDetailRecord record,
            BuildStatus buildStatus
    ) {
        var oldStatus = BuildStatus.parse(record.getStatus());
        if (!oldStatus.isFinish()) {
            return new Pair<>(true, buildStatus);
        } else {
            return new Pair<>(false, oldStatus);
        }
    }

    private void traverseModel(Model model, ModelInterface modelInterface) {
        for (var stage : model.getStages()) {
            var traverse = modelInterface.onFindStage(stage, model);
            if (traverse == Traverse.BREAK) {
                return;
            } else if (traverse == Traverse.SKIP) {
                continue;
            }

            for (var container : stage.getContainers()) {
                var cTraverse = modelInterface.onFindContainer(container, stage);
                if (cTraverse == Traverse.BREAK) {
                    return;
                } else if (cTraverse == Traverse.SKIP) {
                    continue;
                }

                var elements = container.getElements();
                for (int i = 0; i < elements.size(); i++) {
                    if (modelInterface.onFindElement(i, elements.get(i), container) == Traverse.BREAK) {
                        return;
                    }
                }

                if (Boolean.TRUE.equals(container.getMatrixGroupFlag())) {
                    var groupContainers = container.fetchGroupContainers();
                    if (groupContainers != null) {
                        for (var groupContainer : groupContainers) {
                            var gTraverse = modelInterface.onFindContainer(groupContainer, stage);
                            if (gTraverse == Traverse.BREAK) {
                                return;
                            } else if (gTraverse == Traverse.SKIP) {
                                continue;
                            }

                            var groupElements = groupContainer.getElements();
                            for (int i = 0; i < groupElements.size(); i++) {
                                if (modelInterface.onFindElement(i, groupElements.get(i), groupContainer) == Traverse.BREAK) {
                                    return;
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private void pipelineDetailChangeEvent(String projectId, String buildId) {
        var pipelineBuildInfo = pipelineBuildDao.getUserBuildInfo( projectId, buildId);
        if (pipelineBuildInfo != null && pipelineBuildInfo.getChannelCode() == ChannelCode.GIT) {
            pipelineEventDispatcher.dispatch(
                    new PipelineBuildWebSocketPushEvent(
                            "recordDetail",
                            pipelineBuildInfo.getProjectId(),
                            pipelineBuildInfo.getPipelineId(),
                            pipelineBuildInfo.getStartUser(),
                            buildId,
                            RefreshType.DETAIL.getBinary() | RefreshType.RECORD.getBinary(),
                            pipelineBuildInfo.getExecuteCount()
                    )
            );
        }
    }

    protected interface ModelInterface {
        default Traverse onFindStage(Stage stage, Model model) {
            return Traverse.CONTINUE;
        }

        default Traverse onFindContainer(Container container, Stage stage) {
            return Traverse.CONTINUE;
        }

        default Traverse onFindElement(int index, Element e, Container c) {
            return Traverse.CONTINUE;
        }

        boolean needUpdate();
    }

    public enum Traverse {
        BREAK,
        CONTINUE,
        SKIP
    }

    public static class Pair<A, B> {
        private final A first;
        private final B second;

        public Pair(A first, B second) {
            this.first = first;
            this.second = second;
        }

        public A getFirst() {
            return first;
        }

        public B getSecond() {
            return second;
        }
    }
}
