package cd.casic.ci.process.process.service.base;




import cd.casic.ci.common.pipeline.Model;
import cd.casic.ci.common.pipeline.container.Container;
import cd.casic.ci.common.pipeline.container.Stage;
import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.common.pipeline.enums.StartType;
import cd.casic.ci.common.pipeline.pojo.BuildFormProperty;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.element.ElementAdditionalOptions;
import cd.casic.ci.process.api.engine.pojo.BuildInfo;
import cd.casic.ci.process.api.engine.pojo.PipelineInfo;
import cd.casic.ci.process.api.process.pojo.BuildStageStatus;
import cd.casic.ci.process.api.process.utils.PipelineVarUtil;
import cd.casic.ci.process.process.dal.engin.PipelineTriggerReviewDao;
import cd.casic.ci.process.process.dataObject.TPipelineBuildDetailRecord;
import cd.casic.ci.process.process.dataObject.TPipelineBuildSummaryRecord;
import cd.casic.ci.process.process.service.base.dispatcher.PipelineEventDispatcher;
import cd.casic.ci.common.pipeline.utils.ModelUtils;
import cd.casic.ci.process.api.process.pojo.pipeline.ModelDetail;
import cd.casic.ci.process.process.dal.pipeline.PipelineBuildDao;
import cd.casic.ci.process.process.dal.record.BuildDetailDao;
import cd.casic.ci.process.process.dal.record.PipelineBuildSummaryDao;
import cd.casic.ci.process.process.service.base.engin.service.PipelineRepositoryService;
import cd.casic.ci.process.process.service.base.engin.util.ContainerUtils;
import com.alibaba.fastjson.JSON;
import com.github.fppt.jedismock.operations.RedisOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Service
@Slf4j
public class PipelineBuildDetailServiceImpl extends BaseBuildDetailServiceImpl {

    @Value("${pipeline.build.retry.limit_days:21}")
    private int retryLimitDays;

    private final PipelineRepositoryService pipelineRepositoryService;
    private  PipelineBuildSummaryDao pipelineBuildSummaryDao;
    private final PipelineTriggerReviewDao pipelineTriggerReviewDao;

    @Autowired
    public PipelineBuildDetailServiceImpl(
            PipelineRepositoryService pipelineRepositoryService,
            PipelineBuildSummaryDao pipelineBuildSummaryDao,
            PipelineTriggerReviewDao pipelineTriggerReviewDao,
            PipelineBuildDao pipelineBuildDao,
            BuildDetailDao buildDetailDao,
            RedisOperation redisOperation,
            StageTagServiceImpl stageTagServiceImpl,
            PipelineEventDispatcher pipelineEventDispatcher
    ) {
        super(pipelineBuildDao, buildDetailDao, stageTagServiceImpl, pipelineEventDispatcher);
        this.pipelineRepositoryService = pipelineRepositoryService;
        this.pipelineBuildSummaryDao = pipelineBuildSummaryDao;
        this.pipelineTriggerReviewDao = pipelineTriggerReviewDao;
    }

    private boolean checkPassDays(Long startTime) {
        if (retryLimitDays < 0 || startTime == null) {
            return true;
        }
        return (System.currentTimeMillis() - startTime) < TimeUnit.DAYS.toMillis(retryLimitDays);
    }

    /**
     * 查询ModelDetail
     * @param projectId 项目Id
     * @param buildId 构建Id
     * @param refreshStatus 是否刷新状态
     */
    public ModelDetail get(String projectId, String buildId, boolean refreshStatus) {
        TPipelineBuildDetailRecord record = buildDetailDao.get(projectId, buildId);
        if (record == null) {
            return null;
        }

        BuildInfo buildInfo = pipelineBuildDao.getUserBuildInfo(
                projectId,
                buildId
        );
        if (buildInfo == null) {
            return null;
        }

        PipelineInfo pipelineInfo = pipelineRepositoryService.getPipelineInfo(projectId, buildInfo.getPipelineId(),null,false);
        if (pipelineInfo == null) {
            return null;
        }

        TPipelineBuildSummaryRecord buildSummaryRecord = pipelineBuildSummaryDao.get(projectId, buildInfo.getPipelineId());
        Model model = JSON.parseObject(record.getModel(), Model.class);

        if (refreshStatus) {
            if (checkPassDays(buildInfo.getStartTime())) {
                ModelUtils.refreshCanRetry(model);
            }
        }

        var triggerContainer = model.getTriggerContainer();
        if (triggerContainer.getBuildNo() != null) {
            triggerContainer.getBuildNo().setCurrentBuildNo(
                    buildSummaryRecord != null ? buildSummaryRecord.getBuildNo() : triggerContainer.getBuildNo().getBuildNo()
            );
        }

        List<BuildFormProperty> params = triggerContainer.getParams();
        List<BuildFormProperty> newParams = new ArrayList<BuildFormProperty>(params.size());
        params.forEach(it -> {
            var newVarName = PipelineVarUtil.oldVarToNewVar(it.getId());
            if (newVarName != null && !newVarName.isBlank()) {
                BuildFormProperty buildFormProperty = new BuildFormProperty();
                BeanUtils.copyProperties(it,buildFormProperty);
                buildFormProperty.setId(newVarName);
                newParams.add(buildFormProperty);
            } else {
                newParams.add(it);
            }
        });
        triggerContainer.setParams(newParams);

        model.getStages().forEach(stage -> {
            stage.resetBuildOption(false);
            stage.getContainers().forEach(container -> {
                container.setContainerHashId(container.getContainerHashId() != null ?
                        container.getContainerHashId() : container.getContainerId());
                container.setContainerId(container.getId());
            });
        });

        List<String> triggerReviewers = pipelineTriggerReviewDao.getTriggerReviewers(
                projectId,
                pipelineInfo.getPipelineId(),
                buildId
        );

        return new ModelDetail(
                record.getBuildId(),
                buildInfo.getPipelineId(),
                model.getName(),
                record.getStartUser() != null ? record.getStartUser() : "",
                buildInfo.getTriggerUser(),
                StartType.toReadableString(
                        buildInfo.getTrigger(),
                        buildInfo.getChannelCode(),
                        ""
                ),
                record.getStartTime() != null ? record.getStartTime().getTime() :
                        LocalDateTime.now().atOffset(ZoneOffset.UTC).toInstant().toEpochMilli(),
                record.getEndTime() != null ? record.getEndTime().getTime() : null,
                record.getStatus() != null ? record.getStatus() : "",
                model,
                System.currentTimeMillis(),
                buildInfo.getBuildNum(),
                record.getCancelUser() != null ? record.getCancelUser() : "",
                buildInfo.getVersion(),
                pipelineInfo.getVersion(),
                buildSummaryRecord != null ? buildSummaryRecord.getBuildNum() : -1,
                pipelineInfo.getLastModifyUser(),
                buildInfo.getExecuteTime(),
                triggerReviewers,
                buildInfo.isDebug()
        );
    }

    public void updateModel(String projectId, String buildId, Model model) {
        buildDetailDao.update(
                projectId,
                buildId,
                JSON.toJSONString(model, false),
                BuildStatus.RUNNING,
                null
        );
    }

    public void buildCancel(String projectId, String buildId, BuildStatus buildStatus, String cancelUser) {
        log.info("Cancel the build {} by {}", buildId, cancelUser);
        update(
                projectId,
                buildId,
                new ModelInterface() {
                    private boolean update = false;

                    @Override
                    public Traverse onFindStage(Stage stage, Model model) {
                        if (BuildStatus.parse(stage.getStatus()).isRunning()) {
                            stage.setStatus(buildStatus.name());
                            if (stage.getStartEpoch() == null) {
                                stage.setElapsed(0L);
                            } else {
                                stage.setElapsed(System.currentTimeMillis() - stage.getStartEpoch());
                            }
                            update = true;
                        }
                        return Traverse.CONTINUE;
                    }

                    @Override
                    public Traverse onFindContainer(Container container, Stage stage) {
                        var status = BuildStatus.parse(container.getStatus());
                        if (status == BuildStatus.PREPARE_ENV) {
                            if (container.getStartEpoch() == null) {
                                container.setSystemElapsed(0L);
                            } else {
                                container.setSystemElapsed(System.currentTimeMillis() - container.getStartEpoch());
                            }
                            update = true;
                        }

                        var refreshFlag = status.isRunning() &&
                                (container.getElements().get(0).getStatus() == null ||
                                        container.getElements().get(0).getStatus().isBlank()) &&
                                container.getContainPostTaskFlag() != Boolean.TRUE;

                        if (status == BuildStatus.PREPARE_ENV || refreshFlag) {
                            container.setName(ContainerUtils.getClearedQueueContainerName(container.getName()));
                            container.setStatus(buildStatus.name());
                        }
                        return Traverse.CONTINUE;
                    }

                    @Override
                    public Traverse onFindElement(int index, Element e, Container c) {
                        if (e.getStatus() != null &&
                                (e.getStatus().equals(BuildStatus.RUNNING.name()) ||
                                        e.getStatus().equals(BuildStatus.REVIEWING.name()))) {

                            String status;
                            if (e.getStatus().equals(BuildStatus.RUNNING.name())) {
                                var runCondition = e.getAdditionalOptions() != null ?
                                        e.getAdditionalOptions().getRunCondition() : null;
                                if (runCondition == ElementAdditionalOptions.RunCondition.PRE_TASK_FAILED_EVEN_CANCEL) {
                                    status = BuildStatus.RUNNING.name();
                                } else {
                                    status = BuildStatus.CANCELED.name();
                                }
                            } else {
                                status = buildStatus.name();
                            }

                            e.setStatus(status);
                            if (c.getContainPostTaskFlag() != Boolean.TRUE) {
                                c.setStatus(status);
                            }

                            if (BuildStatus.parse(status).isFinish()) {
                                if (e.getStartEpoch() != null) {
                                    e.setElapsed(System.currentTimeMillis() - e.getStartEpoch());
                                }

                                long elementElapsed = 0L;
                                for (var element : c.getElements()) {
                                    elementElapsed += element.getElapsed() != null ? element.getElapsed() : 0;
                                    if (element == e) {
                                        break;
                                    }
                                }
                                c.setElementElapsed(elementElapsed);
                            }
                            update = true;
                        }
                        return Traverse.CONTINUE;
                    }

                    @Override
                    public boolean needUpdate() {
                        return update;
                    }
                },
                BuildStatus.RUNNING,
                cancelUser,
                "buildCancel"
        );
    }

    public Pair<Model, List<BuildStageStatus>> buildEnd(
            String projectId,
            String buildId,
            BuildStatus buildStatus,
            String errorMsg
    ) {
        log.info("[{}]|BUILD_END|buildStatus={}", buildId, buildStatus);
       final List<BuildStageStatus> allStageStatus = new ArrayList<>();

        var model = update(
                projectId,
                buildId,
                new ModelInterface() {
                    private boolean update = false;

                    @Override
                    public Traverse onFindContainer(Container container, Stage stage) {
                        if (BuildStatus.parse(container.getStatus()).isRunning()) {
                            container.setStatus(buildStatus.name());
                            update = true;
                            if (container.getStartEpoch() == null) {
                                container.setElementElapsed(0L);
                            } else {
                                container.setElementElapsed(System.currentTimeMillis() - container.getStartEpoch());
                            }
                            container.setName(ContainerUtils.getClearedQueueContainerName(container.getName()));
                        }
                        return Traverse.CONTINUE;
                    }

                    @Override
                    public Traverse onFindStage(Stage stage, Model model) {
                        if (allStageStatus.isEmpty()) {
                            allStageStatus.addAll(fetchHistoryStageStatus(model, buildStatus,null, errorMsg,null));
                        }
                        if (BuildStatus.parse(stage.getStatus()).isRunning()) {
                            stage.setStatus(buildStatus.name());
                            update = true;
                            if (stage.getStartEpoch() == null) {
                                stage.setElapsed(0L);
                            } else {
                                stage.setElapsed(System.currentTimeMillis() - stage.getStartEpoch());
                            }
                        }
                        return Traverse.CONTINUE;
                    }

                    @Override
                    public Traverse onFindElement(int index, Element e, Container c) {
                        if (e.getStatus() != null && !e.getStatus().isBlank() &&
                                BuildStatus.valueOf(e.getStatus()).isRunning()) {
                            e.setStatus(buildStatus.name());
                            update = true;
                            if (e.getStartEpoch() != null) {
                                e.setElapsed(System.currentTimeMillis() - e.getStartEpoch());
                            }

                            long elementElapsed = 0L;
                            for (var element : c.getElements()) {
                                elementElapsed += element.getElapsed() != null ? element.getElapsed() : 0;
                                if (element == e) {
                                    break;
                                }
                            }
                            c.setElementElapsed(elementElapsed);
                        }
                        return Traverse.CONTINUE;
                    }

                    @Override
                    public boolean needUpdate() {
                        return update;
                    }
                },
                buildStatus,null,
                "buildEnd"
        );
        return new Pair<>(model, allStageStatus);
    }

    public void updateBuildCancelUser(String projectId, String buildId, String cancelUserId) {
        buildDetailDao.updateBuildCancelUser(
                projectId,
                buildId,
                cancelUserId
        );
    }
}