package cd.casic.ci.process.process.service.base;

import cd.casic.ci.common.pipeline.UUIDUtil;
import cd.casic.ci.common.pipeline.constant.CommonMessageCode;
import cd.casic.ci.process.api.process.pojo.PipelineStageTag;
import cd.casic.ci.process.process.dal.pipeline.PipelineStageTagDao;
import cd.casic.ci.process.process.dataObject.TPipelineStageTagRecord;
import com.github.benmanes.caffeine.cache.Caffeine;

import com.github.benmanes.caffeine.cache.LoadingCache;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.util.ArrayList;
import java.util.List;

/**
 * StageTag为后台插入，更新由平台OP控制，基本不会更新，只需要缓存在内存即可。即使出错也不影响正常逻辑，暂不引入redis
 */
@Service
@Slf4j
public class StageTagServiceImpl {

    private static final String ALL = "ALL";
    private static final String DEFAULT = "DEFAULT";
    @Resource
    private PipelineStageTagDao pipelineStageTagDao;
    // TODO 不引入缓存
    // StageTag为后台插入，更新由平台OP控制，不会经常更新，所以只需要缓存在内存即可。即使出错也不影响正常逻辑，暂不引入redis
//    private  LoadingCache<String, PipelineStageTag> defaultTagCache = Caffeine.newBuilder()
//            .expireAfterWrite(Duration.ofMinutes(1))
//            .maximumSize(1)
//            .build(key -> pipelineStageTagDao.getDefaultStageTag());

    // StageTag为后台插入，更新由平台OP控制，不会经常更新，所以只需要缓存在内存即可。即使出错也不影响正常逻辑，暂不引入redis
    private final LoadingCache<String, List<PipelineStageTag>> allTagCache = Caffeine.newBuilder()
            .expireAfterWrite(Duration.ofMinutes(1))
            .maximumSize(1)
            .build(key -> {
                List<PipelineStageTag> pipelineStageTagList = new ArrayList<>();
                List<TPipelineStageTagRecord> allStageTag = pipelineStageTagDao.getAllStageTag();
                for (int i = 0; i < allStageTag.size(); i++) {
                    pipelineStageTagList.add(PipelineStageTagDao.convert(allStageTag.get(i), i == 0));
                }
                return pipelineStageTagList;
            });

    @Autowired
    public StageTagServiceImpl( PipelineStageTagDao pipelineStageTagDao) {

        this.pipelineStageTagDao = pipelineStageTagDao;
    }

    /**
     * 获取所有阶段标签信息
     */
    public List<PipelineStageTag> getAllStageTag() {
        return allTagCache.get(ALL);
    }

//    /**
//     * 获取默认标签
//     */
//    public PipelineStageTag getDefaultStageTag() {
//        return defaultTagCache.get(DEFAULT);
//    }

    /**
     * 根据id获取阶段标签信息
     */
    public PipelineStageTag getStageTag(String id) {
        List<PipelineStageTag> all = allTagCache.getIfPresent(ALL);
        PipelineStageTag data;
        if (all == null || all.isEmpty()) {
            TPipelineStageTagRecord pipelineStageTagRecord = pipelineStageTagDao.getStageTag(id);
            log.info("the pipelineStageTagRecord is :{}", pipelineStageTagRecord);
            data = pipelineStageTagRecord != null ?
                    PipelineStageTagDao.convert(pipelineStageTagRecord, false) : null;
        } else {
            data = all.stream()
                    .filter(tag -> tag.getId().equals(id))
                    .findFirst()
                    .orElse(null);
        }
        return data;
    }

    /**
     * 保存阶段标签信息
     */
    public Boolean saveStageTag(String stageTag, int weight) {
        log.info("the save stageTagName is:{}", stageTag);
        // 判断阶段标签名称是否存在
        int count = getCountByNameOrWeight(stageTag, weight);
        if (count > 0) {
            // 抛出错误提示
            return false;
        }
        String id = UUIDUtil.generate();
        pipelineStageTagDao.add(id, stageTag, weight);
//        defaultTagCache.invalidateAll();
        allTagCache.invalidateAll();
        return true;
    }

    /**
     * 更新阶段标签信息
     */
    public Boolean updateStageTag(String id, String stageTagName, int weight) {
        log.info("the update stageTagName is:{}", stageTagName);
        // 判断阶段标签代码是否存在
        if (getCountByNameOrWeight(stageTagName, weight) > 0) {
            // 判断更新的阶段标签代码是否属于自已
            TPipelineStageTagRecord pipelineStageTag = pipelineStageTagDao.getStageTag(id);
            if (pipelineStageTag != null && !stageTagName.equals(pipelineStageTag.getStageTagName())) {
                // 抛出错误提示
                return false;
            }
        }
        pipelineStageTagDao.update(id, stageTagName, weight);
//        defaultTagCache.invalidateAll();
//        allTagCache.invalidateAll();
        return true;
    }

    /**
     * 删除阶段标签信息
     */
    public Boolean deleteStageTag(String id) {
        log.info("the delete id is :{}", id);
        pipelineStageTagDao.delete( id);
//        defaultTagCache.invalidateAll();
        allTagCache.invalidateAll();
        return true;
    }

    /**
     * 根据阶段标签名称和权重查询重复数据库记录数
     */
    private int getCountByNameOrWeight(String stageTagName, int weight) {
        Integer count = pipelineStageTagDao.countByNameOrWeight(stageTagName, weight);
        int result = 0;
        if (count != null) {
            result = count;
        }
        return result;
    }
}