package cd.casic.ci.process.process.service.base.engin.util;

import cd.casic.ci.common.pipeline.container.Container;
import cd.casic.ci.common.pipeline.container.NormalContainer;
import cd.casic.ci.common.pipeline.container.VMBuildContainer;
import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.process.api.process.constant.ProcessMessageCode;

public  class ContainerUtils {

    private ContainerUtils() {
        // 私有构造函数，防止实例化
    }

    /**
     * 获取容器启动键
     * @param pipelineId 流水线ID
     * @param buildId 构建ID
     * @param containerId 容器ID
     * @return 容器启动键
     */
    public static String getContainerStartupKey(String pipelineId, String buildId, String containerId) {
        return String.format("container:startup:%s:%s:%s", pipelineId, buildId, containerId);
    }

    /**
     * 获取容器运行事件取消任务键
     * @param pipelineId 流水线ID
     * @param buildId 构建ID
     * @param containerId 容器ID
     * @return 容器运行事件取消任务键
     */
    public static String getContainerRunEvenCancelTaskKey(String pipelineId, String buildId, String containerId) {
        return String.format("container:taskEvenCancel:%s:%s:%s", pipelineId, buildId, containerId);
    }

    /**
     * 判断普通容器是否启用
     * @param normalContainer 普通容器
     * @return 是否启用
     */
    public static boolean isNormalContainerEnable(NormalContainer normalContainer) {
        if (normalContainer.getJobControlOption() != null) {
            return normalContainer.getJobControlOption().getEnable();
        }
        return !normalContainer.getEnableSkip();
    }

    /**
     * 判断VM构建容器是否启用
     * @param container VM构建容器
     * @return 是否启用
     */
    public static boolean isVMBuildContainerEnable(VMBuildContainer container) {
        return container.getJobControlOption() == null || container.getJobControlOption().getEnable();
    }

    private static String getMutexPrefix() {
        return ProcessMessageCode.BK_MUTEX_WAITING;
    }

    /**
     * 获取互斥固定容器名称
     * @param containerName 容器名称
     * @return 处理后的容器名称
     */
    public static String getMutexFixedContainerName(String containerName) {
        String prefix = getMutexPrefix();
        if (containerName.startsWith(prefix)) {
            return containerName.substring(prefix.length());
        }
        return containerName;
    }

    /**
     * 获取互斥等待名称
     * @param containerName 容器名称
     * @return 处理后的容器名称
     */
    public static String getMutexWaitName(String containerName) {
        String prefix = getMutexPrefix();
        if (containerName.startsWith(prefix)) {
            return containerName;
        }
        return prefix + containerName;
    }

    private static String getQueuePrefix() {
        return ProcessMessageCode.BK_QUEUING;
    }

    private static String getReviewPrefix() {
        return ProcessMessageCode.BK_PENDING;
    }

    /**
     * 获取清除队列容器名称
     * @param containerName 容器名称
     * @return 处理后的容器名称
     */
    public static String getClearedQueueContainerName(String containerName) {
        String queuePrefix = getQueuePrefix();
        String reviewPrefix = getReviewPrefix();

        if (containerName.startsWith(queuePrefix)) {
            return containerName.substring(queuePrefix.length());
        } else if (containerName.startsWith(reviewPrefix)) {
            return containerName.substring(reviewPrefix.length());
        }
        return containerName;
    }

    /**
     * 获取排队等待名称
     * @param containerName 容器名称
     * @param startBuildStatus 开始构建状态
     * @return 处理后的容器名称
     */
    public static String getQueuingWaitName(String containerName, BuildStatus startBuildStatus) {
        String queuePrefix = getQueuePrefix();
        String reviewPrefix = getReviewPrefix();

        if (containerName.startsWith(queuePrefix) || containerName.startsWith(reviewPrefix)) {
            return containerName;
        }

        if (startBuildStatus == BuildStatus.TRIGGER_REVIEWING) {
            return reviewPrefix + containerName;
        }
        return queuePrefix + containerName;
    }

    /**
     * 判断是否为原始矩阵容器
     * @param container 容器
     * @return 是否为原始矩阵容器
     */
    public static boolean isOriginMatrixContainer(Container container) {
        return container.fetchGroupContainers() != null;
    }
}